package tutorial.module.complex;
import adf.agent.info.*;
import adf.component.module.algorithm.Clustering;
import adf.component.module.complex.BuildingDetector;
import adf.agent.module.ModuleManager;
import adf.agent.develop.DevelopData;
import adf.component.module.complex.RoadDetector;
import rescuecore2.standard.entities.*;
import rescuecore2.worldmodel.EntityID;
import java.util.*;
import tutorial.module.algorithm.KmeansPP;

public class AITRoadDetector extends RoadDetector {
    private Clustering clustering;
    private EntityID result;
    private Random random = new Random(1);
    //土木隊のエンティティIDと探索対象(瓦礫)のエンティティIDの紐付け
    private Map<EntityID,EntityID>  BlockadeRoadTOme2_map = new HashMap<>();

    public AITRoadDetector(
            AgentInfo ai, WorldInfo wi, ScenarioInfo si,
            ModuleManager mm, DevelopData dd)
    {
        super(ai, wi, si, mm, dd);
        String clusteringKey = "SampleSearch.Clustering";
        switch (ai.me().getStandardURN())
        {
            case FIRE_BRIGADE:
                clusteringKey += ".Fire";
                break;
            case AMBULANCE_TEAM:
                clusteringKey += ".Ambulance";
                break;
            case POLICE_FORCE:
                clusteringKey += ".Police";
                break;
        }

        this.clustering =
                mm.getModule(clusteringKey,
                        "adf.sample.module.algorithm.SampleKMeans");
        this.registerModule(this.clustering);
    }

    // Tactics内の2に相当
    @Override
    public EntityID getTarget()
    {
        return this.result;
    }

    /**
     * ●土木隊のメソッド
     * ●土木隊リストの中で、number番目に自分(瓦礫)に近い土木隊を見つける
     * ●※以下の近いとはユークリッド距離的に近いという意味
     * ●戻り値は土木隊エージェント一体
     */
    public EntityID Blockade_close_PoliceForce(List<EntityID> PoliceForce_list,EntityID blockaderoad,int number){
        //土木隊のidと(土木隊-自分(瓦礫)間のユークリッド距離)をmapで紐づける宣言
        Map<EntityID, Double> PoliceForceTOBlockadeRoad_distance_map = new HashMap<>();
        EntityID target_id = PoliceForce_list.get(0);

        //PoliceForce_listにある各要素で(土木隊-自分(瓦礫)間のユークリッド距離)を計算
        // そして、その距離と土木隊idをBlockadeRoadTOme_distance_mapに追加
        for(int i = 0; i < PoliceForce_list.size(); i++){
            double distance = this.worldInfo.getDistance(PoliceForce_list.get(i), blockaderoad);
            PoliceForceTOBlockadeRoad_distance_map.put(PoliceForce_list.get(i),distance);
        }

        //土木隊-自分(瓦礫)間のユークリッド距離のリストを作成
        List<Double> PoliceForceTOBlockade_distance_list = new ArrayList<>(PoliceForceTOBlockadeRoad_distance_map.values());
        //土木隊-自分(瓦礫)間のユークリッド距離のリストを距離が短い順に並び替え
        Collections.sort(PoliceForceTOBlockade_distance_list);

        //土木隊-自分(瓦礫)間のユークリッド距離がnumber番目となる土木隊を戻り値とする
        for(EntityID id: PoliceForceTOBlockadeRoad_distance_map.keySet())
            if(PoliceForceTOBlockadeRoad_distance_map.get(id) == PoliceForceTOBlockade_distance_list.get(number - 1))
                target_id = id;
        return target_id;
    }

    // Tactics内の1に相当
    @Override
    public RoadDetector calc()
    {
        EntityID me = this.agentInfo.getID();
        int idx = this.clustering.getClusterIndex(me);

        //idx番目のクラスターに含まれてるオブジェクトのidの集まりを取得してる。
        Collection<EntityID> cluster =
                this.clustering.getClusterEntityIDs(idx);

        // idx番目のクラスターに含まれてるオブジェクトのidの集まりをListに変換
        List<EntityID> list = new ArrayList<>(cluster);

        //// 擬似乱数を用いてランダムに選択します
        int r = this.random.nextInt(list.size());

        //(デフォルト)
        this.result = null;


        /**
         * 以下全て、土木隊エージェント
         */

        //自分が何者(消防隊？土木隊？救急隊？建物？道路？....)かを知るための準備
        StandardEntity me_entity = this.worldInfo.getEntity(me);
        //自分が土木隊であるか否か
        boolean isPoliceForce = me_entity instanceof PoliceForce;

        if(isPoliceForce) {

            //瓦礫から見て一番近い土木隊を見つけ、
            //その土木隊のエンティティIDと瓦礫のエンティティIDをmapで紐付ける
            for(int i = 0; i < list.size(); i++){
                //listのi番目のEntityIDからEntityを生成
                StandardEntity entity = this.worldInfo.getEntity(list.get(i));

                Road road = (Road)entity;
                if (!road.isBlockadesDefined()) continue;

                List<EntityID> PoliceForce_list = new ArrayList<>();

                for(EntityID id : this.worldInfo.getChanged().getChangedEntities()){
                    StandardEntity Neighbour_entity = this.worldInfo.getEntity(id);
                    if (Neighbour_entity instanceof PoliceForce) {
                        PoliceForce_list.add(id);
                    }
                }

                for(int j = 0; j < road.getBlockades().size(); i++){
                    if(PoliceForce_list.size() >= 1){
                        BlockadeRoadTOme2_map.put(Blockade_close_PoliceForce(PoliceForce_list,(road.getBlockades()).get(j),1),(road.getBlockades()).get(j));
                    }
                }
            }

            //瓦礫から見て一番近い土木隊が撤去してくれる
            if(BlockadeRoadTOme2_map.containsKey(me)){
                this.result = BlockadeRoadTOme2_map.get(me);
            }else {
                this.result = list.get(r);
            }
        }

        return this;
    }
}
